import unittest
if __name__ == '__main__':
    import bootstrap
from lesscpy.lessc.scope import Scope
from lesscpy.plib.identifier import Identifier


class TestIdentifier(unittest.TestCase):
    pass

if __name__ == '__main__':
    unittest.main()
